@@ -1,22 +1,30 @@
 use crate::Slot;
 use std::ops::{Deref, DerefMut};
 
-#[derive(Debug)]
 pub struct PoolValue<T: 'static> {
     slot: Slot<T>,
 }
 
+impl<T> Clone for PoolValue<T>
+where
+    T: Clone,
+{
+    fn clone(&self) -> Self {
+        self.slot.pool().alloc(self.slot.value().clone().unwrap())
+    }
+}
+
 impl<T> PoolValue<T> {
     pub(crate) fn new(slot: Slot<T>) -> Self {
         Self { slot }
     }
 
-    pub fn to_value(&self) -> T {
-        self.slot.to_value()
+    pub fn take_value(&self) -> T {
+        self.slot.take_value()
     }
 
-    pub fn to_boxed_value(&self) -> Box<T> {
-        Box::new(self.to_value())
+    pub fn take_boxed_value(&self) -> Box<T> {
+        Box::new(self.take_value())
     }
 }
 
@@ -66,3 +74,12 @@ impl<T> Drop for PoolValue<T> {
         self.slot.take().recycle()
     }
 }
+
+impl<T> std::fmt::Debug for PoolValue<T>
+where
+    T: std::fmt::Debug,
+{
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        write!(f, "{:?}", self.deref())
+    }
+}
